@@ -13,7 +13,6 @@ import platform
 import shutil
 import socket
 import subprocess
-import sys
 from telnetlib import Telnet
 import tempfile
 import time
@@ -24,16 +23,22 @@ from emulator_geo import EmulatorGeo
 from emulator_screen import EmulatorScreen
 
 
-class LogcatProc(ProcessHandlerMixin):
-    """Process handler for logcat which saves all output to a logfile.
+class LogOutputProc(ProcessHandlerMixin):
+    """
+    Process handler for processes which save all output to a logfile.
+    If no logfile is specified, output will still be consumed to prevent
+    the output pipe's from overflowing.
     """
 
-    def __init__(self, logfile, cmd, **kwargs):
+    def __init__(self, cmd, logfile=None,  **kwargs):
         self.logfile = logfile
         kwargs.setdefault('processOutputLine', []).append(self.log_output)
         ProcessHandlerMixin.__init__(self, cmd, **kwargs)
 
     def log_output(self, line):
+        if not self.logfile:
+            return
+
         f = open(self.logfile, 'a')
         f.write(line + "\n")
         f.flush()
@@ -43,10 +48,13 @@ class Emulator(object):
 
     deviceRe = re.compile(r"^emulator-(\d+)(\s*)(.*)$")
     _default_res = '320x480'
+    prefs = {'app.update.enabled': False,
+             'app.update.staging.enabled': False,
+             'app.update.service.enabled': False}
 
     def __init__(self, homedir=None, noWindow=False, logcat_dir=None,
                  arch="x86", emulatorBinary=None, res=None, sdcard=None,
-                 userdata=None):
+                 symbols_path=None, userdata=None):
         self.port = None
         self.dm = None
         self._emulator_launched = False
@@ -67,6 +75,7 @@ class Emulator(object):
         self.screen = EmulatorScreen(self)
         self.homedir = homedir
         self.sdcard = sdcard
+        self.symbols_path = symbols_path
         self.noWindow = noWindow
         if self.homedir is not None:
             self.homedir = os.path.expanduser(homedir)
@@ -74,7 +83,8 @@ class Emulator(object):
         self.copy_userdata = self.dataImg is None
 
     def _check_for_b2g(self):
-        self.b2g = B2GInstance(homedir=self.homedir, emulator=True)
+        self.b2g = B2GInstance(homedir=self.homedir, emulator=True,
+                               symbols_path=self.symbols_path)
         self.adb = self.b2g.adb_path
         self.homedir = self.b2g.homedir
 
@@ -157,13 +167,11 @@ class Emulator(object):
         closed), and self.proc.poll() is also not None (meaning the emulator
         process has terminated).
         """
-        if (self._emulator_launched and self.proc is not None
-                                    and self.proc.poll() is not None):
-            return True
-        return False
+        return self._emulator_launched and self.proc is not None \
+                                       and self.proc.poll() is not None
 
-    def check_for_minidumps(self, symbols_path):
-        return self.b2g.check_for_crashes(symbols_path)
+    def check_for_minidumps(self):
+        return self.b2g.check_for_crashes()
 
     def create_sdcard(self, sdcard):
         self._tmp_sdcard = tempfile.mktemp(prefix='sdcard')
@@ -208,8 +216,7 @@ class Emulator(object):
 
     def close(self):
         if self.is_running and self._emulator_launched:
-            self.proc.terminate()
-            self.proc.wait()
+            self.proc.kill()
         if self._adb_started:
             self._run_adb(['kill-server'])
             self._adb_started = False
@@ -272,6 +279,9 @@ waitFor(
             # older emulators.  45s *should* be enough of a delay
             # to allow telephony API's to work.
             pass
+        except InvalidResponseException:
+            self.check_for_minidumps()
+            raise
         print 'done'
         marionette.set_context(marionette.CONTEXT_CONTENT)
         marionette.delete_session()
@@ -312,9 +322,14 @@ waitFor(
 
         original_online, original_offline = self._get_adb_devices()
 
-        self.proc = subprocess.Popen(qemu_args,
-                                     stdout=subprocess.PIPE,
-                                     stderr=subprocess.PIPE)
+        filename = None
+        if self.logcat_dir:
+            filename = os.path.join(self.logcat_dir, 'qemu.log')
+            if os.path.isfile(filename):
+                self.rotate_log(filename)
+
+        self.proc = LogOutputProc(qemu_args, filename)
+        self.proc.run()
 
         online, offline = self._get_adb_devices()
         now = datetime.datetime.now()
@@ -340,6 +355,28 @@ waitFor(
         # setup DNS fix for networking
         self._run_adb(['shell', 'setprop', 'net.dns1', '10.0.2.3'])
 
+    def wait_for_homescreen(self, marionette):
+        print 'waiting for homescreen...'
+
+        created_session = False
+        if not marionette.session:
+            marionette.start_session()
+            created_session = True
+
+        marionette.set_context(marionette.CONTEXT_CONTENT)
+        marionette.execute_async_script("""
+log('waiting for mozbrowserloadend');
+window.addEventListener('mozbrowserloadend', function loaded(aEvent) {
+  log('received mozbrowserloadend for ' + aEvent.target.src);
+  if (aEvent.target.src.indexOf('ftu') != -1 || aEvent.target.src.indexOf('homescreen') != -1) {
+    window.removeEventListener('mozbrowserloadend', loaded);
+    marionetteScriptFinished();
+  }
+});""", script_timeout=120000)
+        print '...done'
+        if created_session:
+            marionette.delete_session()
+
     def setup(self, marionette, gecko_path=None, busybox=None):
         if busybox:
             self.install_busybox(busybox)
@@ -348,6 +385,27 @@ waitFor(
             self.install_gecko(gecko_path, marionette)
 
         self.wait_for_system_message(marionette)
+        self.set_prefs(marionette)
+
+    def set_prefs(self, marionette):
+        marionette.start_session()
+        marionette.set_context(marionette.CONTEXT_CHROME)
+        for pref in self.prefs:
+            marionette.execute_script("""
+            Components.utils.import("resource://gre/modules/Services.jsm");
+            let argtype = typeof(arguments[1]);
+            switch(argtype) {
+                case 'boolean':
+                    Services.prefs.setBoolPref(arguments[0], arguments[1]);
+                    break;
+                case 'number':
+                    Services.prefs.setIntPref(arguments[0], arguments[1]);
+                    break;
+                default:
+                    Services.prefs.setCharPref(arguments[0], arguments[1]);
+            }
+            """, [pref, self.prefs[pref]])
+        marionette.delete_session()
 
     def restart_b2g(self):
         print 'restarting B2G'
@@ -415,8 +473,14 @@ waitFor(
         """ Rotate a logfile, by recursively rotating logs further in the sequence,
             deleting the last file if necessary.
         """
-        destlog = os.path.join(self.logcat_dir, 'emulator-%d.%d.log' % (self.port, index))
-        if os.access(destlog, os.F_OK):
+        basename = os.path.basename(srclog)
+        basename = basename[:-len('.log')]
+        if index > 1:
+            basename = basename[:-len('.1')]
+        basename = '%s.%d.log' % (basename, index)
+
+        destlog = os.path.join(self.logcat_dir, basename)
+        if os.path.isfile(destlog):
             if index == 3:
                 os.remove(destlog)
             else:
@@ -427,27 +491,25 @@ waitFor(
         """ Save the output of logcat to a file.
         """
         filename = os.path.join(self.logcat_dir, "emulator-%d.log" % self.port)
-        if os.access(filename, os.F_OK):
+        if os.path.isfile(filename):
             self.rotate_log(filename)
-        cmd = [self.adb, '-s', 'emulator-%d' % self.port, 'logcat']
+        cmd = [self.adb, '-s', 'emulator-%d' % self.port, 'logcat', '-v', 'threadtime']
 
-        self.logcat_proc = LogcatProc(filename, cmd)
+        self.logcat_proc = LogOutputProc(cmd, filename)
         self.logcat_proc.run()
 
     def setup_port_forwarding(self, remote_port):
         """ Set up TCP port forwarding to the specified port on the device,
             using any availble local port, and return the local port.
         """
-
-        import socket
         s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
         s.bind(("",0))
         local_port = s.getsockname()[1]
         s.close()
 
-        output = self._run_adb(['forward',
-                                'tcp:%d' % local_port,
-                                'tcp:%d' % remote_port])
+        self._run_adb(['forward',
+                       'tcp:%d' % local_port,
+                       'tcp:%d' % remote_port])
 
         self.marionette_port = local_port
 
