@@ -42,10 +42,17 @@ class MarionetteTestResult(unittest._TextTestResult, TestResultCollection):
     def __init__(self, *args, **kwargs):
         self.marionette = kwargs.pop('marionette')
         TestResultCollection.__init__(self, 'MarionetteTest')
-        unittest._TextTestResult.__init__(self, *args, **kwargs)
         self.passed = 0
         self.testsRun = 0
         self.result_modifiers = [] # used by mixins to modify the result
+        pid = kwargs.pop('b2g_pid')
+        if pid:
+            if B2GTestResultMixin not in self.__class__.__bases__:
+                bases = [b for b in self.__class__.__bases__]
+                bases.append(B2GTestResultMixin)
+                self.__class__.__bases__ = tuple(bases)
+            B2GTestResultMixin.__init__(self, b2g_pid=pid)
+        unittest._TextTestResult.__init__(self, *args, **kwargs)
 
     @property
     def skipped(self):
@@ -240,17 +247,30 @@ class MarionetteTextTestRunner(unittest.TextTestRunner):
     def __init__(self, **kwargs):
         self.marionette = kwargs['marionette']
         self.capabilities = kwargs.pop('capabilities')
+        self.pre_run_functions = []
+        self.b2g_pid = None
         del kwargs['marionette']
+
+        if self.capabilities['device'] != 'desktop' and self.capabilities['b2g']:
+            def b2g_pre_run():
+                dm_type = os.environ.get('DM_TRANS', 'adb')
+                if dm_type == 'adb':
+                    self.b2g_pid = get_b2g_pid(get_dm(self.marionette))
+            self.pre_run_functions.append(b2g_pre_run)
+
         unittest.TextTestRunner.__init__(self, **kwargs)
 
     def _makeResult(self):
         return self.resultclass(self.stream,
                                 self.descriptions,
                                 self.verbosity,
-                                marionette=self.marionette)
+                                marionette=self.marionette,
+                                b2g_pid=self.b2g_pid)
 
     def run(self, test):
         "Run the given test case or test suite."
+        for pre_run_func in self.pre_run_functions:
+            pre_run_func()
         result = self._makeResult()
         if hasattr(self, 'failfast'):
             result.failfast = self.failfast
@@ -311,40 +331,6 @@ class MarionetteTextTestRunner(unittest.TextTestRunner):
         return result
 
 
-class B2GMarionetteTestResult(MarionetteTestResult, B2GTestResultMixin):
-
-    def __init__(self, *args, **kwargs):
-        # stupid hack because _TextTestRunner doesn't accept **kwargs
-        b2g_pid = kwargs.pop('b2g_pid')
-        MarionetteTestResult.__init__(self, *args, **kwargs)
-        kwargs['b2g_pid'] = b2g_pid
-        B2GTestResultMixin.__init__(self, *args, **kwargs)
- 
-
-class B2GMarionetteTextTestRunner(MarionetteTextTestRunner):
-
-    resultclass = B2GMarionetteTestResult
-
-    def __init__(self, **kwargs):
-        MarionetteTextTestRunner.__init__(self, **kwargs)
-        if self.capabilities['device'] != 'desktop':
-            self.resultclass = B2GMarionetteTestResult
-        self.b2g_pid = None
-
-    def _makeResult(self):
-        return self.resultclass(self.stream,
-                                self.descriptions,
-                                self.verbosity,
-                                marionette=self.marionette,
-                                b2g_pid=self.b2g_pid)
-
-    def run(self, test):
-        dm_type = os.environ.get('DM_TRANS', 'adb')
-        if dm_type == 'adb':
-            self.b2g_pid = get_b2g_pid(get_dm(self.marionette))
-        return super(B2GMarionetteTextTestRunner, self).run(test)
-
-
 class BaseMarionetteOptions(OptionParser):
     def __init__(self, **kwargs):
         OptionParser.__init__(self, **kwargs)
@@ -764,9 +750,6 @@ class BaseMarionetteTestRunner(object):
             if not self._capabilities:
                 self.capabilities
 
-        if self.capabilities['device'] != 'desktop':
-            self.textrunnerclass = B2GMarionetteTextTestRunner
-
         for test in tests:
             self.add_test(test)
 
